/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.entity;

import com.google.common.base.Optional;
import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.crash.CrashReportCategory;
import net.minecraft.entity.Entity;
import net.minecraft.entity.MoverType;
import net.minecraft.init.Blocks;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTUtil;
import net.minecraft.network.datasync.DataParameter;
import net.minecraft.network.datasync.DataSerializers;
import net.minecraft.network.datasync.EntityDataManager;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import net.dries007.tfc.util.IFallingBlock;

/**
 * todo: add entity damage
 */
public class EntityFallingBlockTFC extends Entity
{
    private static final DataParameter<BlockPos> ORIGIN = EntityDataManager.func_187226_a(EntityFallingBlockTFC.class, DataSerializers.field_187200_j);
    private static final DataParameter<Optional<IBlockState>> BLOCK = EntityDataManager.func_187226_a(EntityFallingBlockTFC.class, DataSerializers.field_187197_g);

    private IFallingBlock falling;
    private int fallTime;
    private NBTTagCompound teData;

    @SuppressWarnings("unused")
    public EntityFallingBlockTFC(World worldIn)
    {
        super(worldIn);
    }

    public EntityFallingBlockTFC(World world, BlockPos start, IFallingBlock falling, IBlockState state)
    {
        this(world);
        this.falling = falling;
        func_70105_a(0.98F, 0.98F);
        func_70107_b(start.func_177958_n() + 0.5f, start.func_177956_o() + (double) ((1.0F - field_70131_O) / 2.0F), start.func_177952_p() + 0.5f);
        field_70159_w = 0.0D;
        field_70181_x = 0.0D;
        field_70179_y = 0.0D;
        field_70169_q = field_70165_t;
        field_70167_r = field_70163_u;
        field_70166_s = field_70163_u;
        field_70180_af.func_187227_b(ORIGIN, start);
        field_70180_af.func_187227_b(BLOCK, Optional.of(state));
    }

    public IBlockState getState()
    {
        return field_70180_af.func_187225_a(BLOCK).orNull();
    }

    public BlockPos getOrigin()
    {
        return field_70180_af.func_187225_a(ORIGIN);
    }

    @Override
    protected void func_70088_a()
    {
        field_70180_af.func_187214_a(ORIGIN, BlockPos.field_177992_a);
        field_70180_af.func_187214_a(BLOCK, Optional.absent());
    }

    public void func_70071_h_()
    {
        IBlockState state = getState();
        if (state == null) return;

        Block block = state.func_177230_c();

        field_70169_q = field_70165_t;
        field_70167_r = field_70163_u;
        field_70166_s = field_70161_v;

        // First tick only, make block air if it's still the same block we started from otherwise, die.
        if (fallTime++ == 0)
        {
            BlockPos pos = new BlockPos(this);
            if (field_70170_p.func_180495_p(pos) == state)
            {
                field_70170_p.func_175698_g(pos);
            }
            else if (!field_70170_p.field_72995_K)
            {
                func_70106_y();
                return;
            }
        }

        if (!func_189652_ae())
        {
            field_70181_x -= 0.03999999910593033D;
        }

        func_70091_d(MoverType.SELF, field_70159_w, field_70181_x, field_70179_y);

        field_70159_w *= 0.9800000190734863D;
        field_70181_x *= 0.9800000190734863D;
        field_70179_y *= 0.9800000190734863D;

        if (field_70170_p.field_72995_K) return;

        final BlockPos pos = new BlockPos(this); // Post move position

        if (!field_70122_E) // Still falling
        {
            if (fallTime > 100 && (pos.func_177956_o() < 1 || pos.func_177956_o() > 256) || fallTime > 600)
            {
                if (field_70170_p.func_82736_K().func_82766_b("doEntityDrops"))
                {
                    falling.getDropsFromFall(field_70170_p, pos, state, teData, fallTime, field_70143_R).forEach(x -> func_70099_a(x, 0));
                }
                func_70106_y();
            }
        }
        else // On ground
        {
            final IBlockState current = field_70170_p.func_180495_p(pos);

//                if (world.isAirBlock(new BlockPos(posX, posY - 0.009999999776482582D, posZ))) // todo: is a forge fix, what does it do?
            if (falling.canFallThrough(field_70170_p.func_180495_p(new BlockPos(field_70165_t, field_70163_u - 0.009999999776482582D, field_70161_v))))
            {
                field_70122_E = false;
                return;
            }

            field_70159_w *= 0.699999988079071D;
            field_70179_y *= 0.699999988079071D;
            field_70181_x *= -0.5D;

            if (current.func_177230_c() == Blocks.field_180384_M) return;

            func_70106_y();

            if (field_70170_p.func_190527_a(block, pos, true, EnumFacing.UP, null) && !falling.canFallThrough(field_70170_p.func_180495_p(pos.func_177982_a(0, -1, 0))) && field_70170_p.func_180501_a(pos, state, 3))
            {
                falling.onEndFalling(field_70170_p, pos, state, current);

                // Copy all TE data over default data (except pos[X,Y,Z]) if the TE is there. This is vanilla code.
                if (teData != null && block.hasTileEntity(state))
                {
                    TileEntity te = field_70170_p.func_175625_s(pos);
                    if (te != null)
                    {
                        NBTTagCompound currentTeData = te.func_189515_b(new NBTTagCompound());
                        for (String s : teData.func_150296_c())
                        {
                            if (!"x".equals(s) && !"y".equals(s) && !"z".equals(s))
                                currentTeData.func_74782_a(s, teData.func_74781_a(s).func_74737_b());
                        }
                        te.func_145839_a(currentTeData);
                        te.func_70296_d();
                    }
                }
            }
            else if (field_70170_p.func_82736_K().func_82766_b("doEntityDrops"))
            {
                falling.getDropsFromFall(field_70170_p, pos, state, teData, fallTime, field_70143_R).forEach(x -> func_70099_a(x, 0));
            }
        }
    }

    @Override
    protected boolean func_70041_e_()
    {
        return false;
    }

    @Override
    public boolean func_70067_L()
    {
        return !field_70128_L;
    }

    @Override
    protected void func_70037_a(NBTTagCompound compound)
    {
        if (compound.func_74764_b("State"))
            field_70180_af.func_187227_b(BLOCK, Optional.of(NBTUtil.func_190008_d(compound.func_74775_l("State"))));
        fallTime = compound.func_74762_e("FallTime");
        if (compound.func_74764_b("TileEntityData")) teData = compound.func_74775_l("TileEntityData");
    }

    @Override
    protected void func_70014_b(NBTTagCompound compound)
    {
        IBlockState state = getState();
        if (state != null) compound.func_74782_a("State", NBTUtil.func_190009_a(new NBTTagCompound(), state));
        compound.func_74768_a("FallTime", fallTime);
        if (teData != null) compound.func_74782_a("TileEntityData", teData);
    }

    @Override
    public boolean func_70075_an()
    {
        return false;
    }

    @Override
    public void func_85029_a(CrashReportCategory category)
    {
        super.func_85029_a(category);

        category.func_71507_a("Origin", getOrigin());
        category.func_71507_a("State", getState());
        category.func_71507_a("FallTime", fallTime);
        category.func_71507_a("TileEntityData", teData);
    }

    @SideOnly(Side.CLIENT)
    @Override
    public boolean func_90999_ad()
    {
        return false;
    }

    @Override
    public boolean func_184213_bq()
    {
        return true;
    }
}
